//
// Generated from archetype; please customize.
//

package org.codehaus.xma.mojo.packager
import org.apache.maven.plugin.Mojo;

import org.codehaus.groovy.maven.mojo.GroovyMojo
import org.codehaus.plexus.component.configurator.ComponentConfigurator;
import org.codehaus.plexus.component.configurator.expression.ExpressionEvaluator;
import org.codehaus.plexus.configuration.PlexusConfiguration;
import org.codehaus.plexus.configuration.xml.XmlPlexusConfiguration;
import org.codehaus.plexus.util.ReflectionUtils;
import org.codehaus.plexus.util.xml.Xpp3Dom;
import org.codehaus.plexus.util.xml.Xpp3DomBuilder;

import org.apache.tools.ant.types.FileSet;
import org.apache.tools.ant.DirectoryScanner;
import java.util.HashSet;


/**
 * Packager Mojo used to wrap and provide the existing xma ant facility as maven plugin.
 * <p>
 * The components to be packaged can be provided using four different methods:
 * <ul>
 * <li>Using the <xmp><components><xmp> element with a list of componentName=componentPackage style values
 *      (e.g. customeredit=org/openxma/dsl/reference/xma/customeredit/)
 * <li>Using the <xmp><components><xmp> element with a list of componentPackage values (e.g.
 *      org/openxma/dsl/reference/xma/customeredit/)
 * <li>Omitting the components element and defining an xmaXmlFilter element with an ANT fileset style filter value
 * <li>Omitting any of these, which results in packaging all XMA components in the source folder.
 * </ul>
 * <p>
 * Providing a filter as well as excplicit components results in packaging the components first,
 * then searching for further components using the filter and packageing those, too. The filter may
 * also include the explicitely defined components.
 * <p>
 * An example with all the possibilities used:
 * <p>
 * <xmp>
                    <xmaXmlFilter>**&amp;#047;*.xma</xmaXmlFilter>
                    <components>
                        <component>customeredit=org/openxma/dsl/reference/xma/customeredit/</component>
                        <component>org/openxma/dsl/reference/xma/customeredit2/</component>
                    </components>

 <xmp>
 * @goal package
 * @requiresDependencyResolution
 */
class PackageMojo extends GroovyMojo {
    /**
     * @parameter expression="${project}"
     * @required
     * @readonly
    */
    org.apache.maven.project.MavenProject project
   /**
     * @parameter expression="${session}"
     * @required
     * @readonly
    */
    org.apache.maven.execution.MavenSession session

	/**
     * @parameter expression="${project.artifactId}"
     */
    private String appDescriptorDirectory

    /**
     * @parameter
     */
    private String xmaXmlFilter

    /**
     * @parameter expression="xma-app.xml"
     */
    private String appDescriptorFileName
    /**
     * @parameter
     * @optional
     */
    private List components
    /**
     * @parameter
     */
    private List clientLibraries
    /**
     * @parameter
     * @required
     */
    private List clientIncludes
    /**
     * @parameter default-value="false"
     */
    private boolean packAll

    /**
     * @parameter expression="${project.build.directory}/xma/components"
     */
    private String componentsDirectory
    /**
     * @parameter expression="${project.build.directory}/xma/dependencies"
     */
    private String dependenciesDirectory
    /**
     * @parameter expression="${project.build.directory}/xma/web"
     */
    private String xmaWebDirectory
    /**
     * @parameter expression="${project.build.directory}/xma/web/components"
     */
    private String webComponentsDirectory
    /**
     * @parameter expression="${project.build.directory}/xma/web/clientrt"
     */
    private String webClientRtDirectory

    void execute() {
		//removing all registered build listeners, including default (that writes to console)
    	ant.project.getBuildListeners().each {
        	ant.project.removeBuildListener(it)
    	}
		// some preconditions..
		if (project.packaging!=/war/) {
			fail ("not a web project")
		}
		boolean fastDevelopmentMode = session.goals.any {it == "jetty:run" } || System.env["boot.fdm"]
		if (fastDevelopmentMode) {
	  		log.info  "openXma Fast-Development-Mode enabled"
		}
		// check appDescriptor
		def appDescriptorFile = new File(appDescriptorDirectory,appDescriptorFileName)
		if (!appDescriptorFile.exists()) {
			fail ("required appDescriptorFile '${appDescriptorFile.name}' not found in directory '${appDescriptorDirectory}'")
		}
        createBuildDirectories()
        copyDependencyArtifacts()
        jarComponentsAndCreateChecksum(fastDevelopmentMode)
	    jarClientIncludes()
        copyComponentsAndApplicationDescriptor(fastDevelopmentMode)

    }

    String createBuildDirectories() {
        log.info  "${project}"
        ant.mkdir(dir: dependenciesDirectory)
        ant.mkdir(dir: componentsDirectory)
		ant.mkdir(dir: "${webComponentsDirectory}")
		ant.mkdir(dir: "${webClientRtDirectory}")
        ant.taskdef(name: "xmachecksum", classname: "at.spardat.xma.boot.antext.XMAChecksum")
        ant.taskdef(name: "appdes",		 classname: "at.spardat.xma.boot.antext.AppDescriptor")
    }

    void copyDependencyArtifacts() {
        log.info "Package xma runtime dependencies.."
        project.dependencyArtifacts.plus(project.artifacts).unique().
            findAll({artifact -> !["test"].contains(artifact.scope) && !artifact.file.name.endsWith(".war") && !artifact.file.directory}).each({artifact ->
			def fileNameAndExtension = artifact.file.name.split('\\.') as List
			def fileName             = fileNameAndExtension.first()
			def fileNameExtension    = fileNameAndExtension.last()
			if (fileName.split('-').size() <= 1) {
				log.warn("Artifact ${artifact.file.name} does not contain a valid '-' separator between artifactname and version!")
			}
			def fileNameWithoutVersion=null
			fileName.split('-').each({fileNamePart ->
			     if (!fileNamePart.isInteger()) {
			         fileNameWithoutVersion = null==fileNameWithoutVersion?fileNamePart:fileNameWithoutVersion+'-'+fileNamePart
			}
			})
			// log.info "Copy ${dependenciesDirectory}/${fileNameWithoutVersion}.${fileNameExtension}"
			ant.copy(tofile: "${dependenciesDirectory}/${fileNameWithoutVersion}.${fileNameExtension}",
				file:"${artifact.file}")
        })
    }

    void jarComponentsAndCreateChecksum(fastDevelopmentMode) {
        if (!fastDevelopmentMode) {
            def processedComponents = new HashSet();
            def componentNames = new HashSet();

            log.info "Copy component jars.."
			components.each({ componentNameAndPackage ->
				def componentSplitted = componentNameAndPackage.split('=')

				def componentName
				def componentPackage
				if (componentSplitted.size() > 1) {
    				componentName = componentSplitted[0]
    				componentPackage = componentSplitted[1]
				} else {
                    componentPackage = componentSplitted[0]
                    def componentPath = componentNameAndPackage.split('/')
                    componentName = componentPath[componentPath.length - 1]
				}

				if (componentPackage.endsWith('/')) {
				    componentPackage = componentPackage.substring(0, componentPackage.length() - 1 );
				}

				packComponent(componentName, componentPackage)
				processedComponents.add(componentPackage);
				if (componentNames.contains(componentName)) {
				    fail("Component name is not unique '${componentName}', the component must be renamed.")
				}
				componentNames.add(componentName);
			})

			if (components == null || components.isEmpty() || xmaXmlFilter != null) {
			    if (xmaXmlFilter == null) {
			        xmaXmlFilter = "**/*.xma";
			    }
                log.info "Searching for xma components with: '${xmaXmlFilter}'"

	            FileSet fileSet = new FileSet();
	            fileSet.setProject(ant.project);
                fileSet.setDir(new File( project.build.sourceDirectory ));
                fileSet.setIncludes(xmaXmlFilter);

                DirectoryScanner ds = fileSet.getDirectoryScanner(ant.project);
                File dir = ds.getBasedir();

                String[] filesInSet = ds.getIncludedFiles();
                for (int i = 0; i < filesInSet.length; i++) {
                    def compXmaName = filesInSet[i]
                    compXmaName = compXmaName.replace('\\', '/')

                    log.info "Found xma component xml: '${compXmaName}'"

                    def componentPackage = compXmaName.substring(0,compXmaName.lastIndexOf('/'))
                    def componentName = compXmaName.substring(compXmaName.lastIndexOf('/')+1	,compXmaName.lastIndexOf('.'))

                    if (processedComponents.contains(componentPackage)) {
                        log.info "Component already packed, skipping now: '${componentPackage}'"
                    } else {
                        packComponent(componentName, componentPackage);
                        processedComponents.add(componentPackage);

                        if (componentNames.contains(componentName)) {
                            fail("Component name is not unique '${componentName}', the component must be renamed.")
                        }
                        componentNames.add(componentName);
                    }
                }
			}
            if (componentNames.isEmpty()) {
                fail("No component has been packed. This might not be what you want, so you maybe want to check your config.")
            }
		}
        log.info "Copy client libraries.."
		clientLibraries.each({ jarNameAndDestinationDirectory ->
			def splitted = jarNameAndDestinationDirectory.split('=')
    		def libraryName = splitted[0]
    		def destinationFolder = splitted[1]
			def destFolderFile = new File(xmaWebDirectory, destinationFolder)
			if (!destFolderFile.exists()) {
				ant.mkdir(dir: xmaWebDirectory + File.separator + destinationFolder)
			}
			ant.copy(tofile: "${xmaWebDirectory}/${destinationFolder}/${libraryName}.jar", file:"${dependenciesDirectory}/${libraryName}.jar")
			ant.xmachecksum(file: "${xmaWebDirectory}/${destinationFolder}/${libraryName}.jar")
		})
    }

    void packComponent(componentName, componentPackage) {
        log.info "Package client/server component '${componentName}'.."
        ant.jar(
            destfile: "${componentsDirectory}/${componentName}client.jar",
            basedir:  "${project.build.outputDirectory}",
            includes: "${componentPackage}/**",
            excludes: "${componentPackage}/**/server/*.*"
        )
        ant.xmachecksum(file: "${componentsDirectory}/${componentName}client.jar")
        ant.jar(
            destfile: "${componentsDirectory}/${componentName}server.jar",
            basedir:  "${project.build.outputDirectory}",
            includes: "${componentPackage}/**",
            excludes: "${componentPackage}/**/client/*.*"
        )
    }

    void jarClientIncludes() {
        log.info "Package common client includes.."
		clientIncludes.each({ clientIncludePath -> log.info clientIncludePath })
            ant.jar(destfile: "${componentsDirectory}/commonclient.jar", update: true) {
                fileset(dir: "${project.build.outputDirectory}", includes: "${clientIncludes.join(',')}")
        }
        ant.xmachecksum(file: "${componentsDirectory}/commonclient.jar")
    }

    void copyComponentsAndApplicationDescriptor(fastDevelopmentMode) {
        log.info "Copy components jars.."
        if (new File("${dependenciesDirectory}/xmabootrt.zip" ).exists()) {
            ant.unzip(src:"${dependenciesDirectory}/xmabootrt.zip" ,dest:"${dependenciesDirectory}")
        }
		ant.copy(todir: "${webComponentsDirectory}", preservelastmodified:"true") {
			fileset(dir: "${componentsDirectory}", includes:"*.jar*", excludes:"*server.jar*")
		}
		ant.copy(todir : "${webClientRtDirectory}",	file : "${dependenciesDirectory}/xmartclient.jar",preservelastmodified : "true"	)
		ant.xmachecksum(file: "${webClientRtDirectory}/xmartclient.jar")

		ant.copy(todir : "${webClientRtDirectory}",	file : "${dependenciesDirectory}/epclient.jar",preservelastmodified : "true"	)
		ant.xmachecksum(file: "${webClientRtDirectory}/epclient.jar")

		log.info "Prepare xma application descriptor.."
		ant.copy(todir : "${xmaWebDirectory}", preservelastmodified : "true") {
            fileset(dir: "${appDescriptorDirectory}", includes:"**/*.*")
        }
		ant.appdes(dir: "${xmaWebDirectory}", file: "${appDescriptorFileName}", fdm: fastDevelopmentMode)
    }
}


